VERSION 1.0 CLASS
BEGIN
  MultiUse = 0   'False
END
Attribute VB_Name = "UncConversion"
Attribute VB_Creatable = True
Attribute VB_Exposed = True
Attribute VB_HelpID = 1
Attribute VB_Description = "Convert pathed path (ie, X:) to a UNC path name (ie, \\\\MyServer\\MyShare)."
' ------------------------------------------------------
' UNCConvt.CLS -- Copyright (c) Slightly Tilted Software
' By:       L.J. Johnson
' Date:     10-13-1996
' Comments: Class module with one exposed method,
'           GetUNCName.
' ------------------------------------------------------
Option Explicit

Private Const BASE_ERR_NUM = vbObjectError + 1048

Private Const NO_ERROR = 0&
Private Const ERROR_MORE_DATA = 234&

Private Declare Sub CopyMem _
    Lib "kernel32" Alias "RtlMoveMemory" _
    (lpTo As Any, _
     lpFrom As Any, _
     ByVal plngCount As Long)
Private Declare Function lstrlen _
    Lib "kernel32" Alias "lstrlenA" _
    (ByVal sText As Any) As Long
Private Declare Function WNetOpenEnum _
   Lib "mpr.dll" Alias "WNetOpenEnumA" _
   (ByVal dwScope As Long, _
    ByVal dwType As Long, _
    ByVal dwUsage As Long, _
    lpNetResource As Any, _
    lphEnum As Long) As Long
Private Declare Function WNetEnumResource _
   Lib "mpr.dll" Alias "WNetEnumResourceA" _
   (ByVal hEnum As Long, _
    lpcCount As Long, _
    lpBuffer As Any, _
    lpBufferSize As Long) As Long
Private Declare Function WNetCloseEnum _
   Lib "mpr.dll" _
   (ByVal hEnum As Long) As Long

Private Const RESOURCE_CONNECTED = &H1&
Private Const RESOURCETYPE_ANY = &H0&
Private Const RESOURCEUSAGE_ALL = &H0&
Private Const RESOURCE_ENUM_ALL = &HFFFF&

Private Type NETRESOURCE
    dwScope          As Long
    dwType           As Long
    dwDisplayType    As Long
    dwUsage          As Long
    pLocalName       As Long
    pRemoteName      As Long
    pComment         As Long
    pProvider        As Long
End Type

Private Type NETRESOURCE_REAL
    dwScope          As Long
    dwType           As Long
    dwDisplayType    As Long
    dwUsage          As Long
    sLocalName       As String
    sRemoteName      As String
    sComment         As String
    sProvider        As String
End Type

Private Const DRIVE_REMOTE = 4&
Private Declare Function GetDriveType _
   Lib "kernel32" Alias "GetDriveTypeA" _
   (ByVal nDrive As String) As Long

' ------------------------------------------------
'
' ------------------------------------------------
Public Function GetUNCName(xstrPassed As String) As String
Attribute GetUNCName.VB_HelpID = 2
Attribute GetUNCName.VB_Description = "Passed the drive-based path, returns the UNC-based  path."
   Dim plngRtn          As Long
   Dim hEnum            As Long
   Dim plngCount        As Long
   Dim plngLoop         As Long
   Dim plngLen          As Long
   Dim plngBufSize      As Long
   Dim patypNetApi()    As NETRESOURCE
   Dim patypNet()       As NETRESOURCE_REAL
   Dim pabytBuf()       As Byte
   Dim pstrStart        As String
   Dim pstrRemaining    As String
   Dim pbolRtnUnc       As Boolean
   Dim plngPos          As Long
   Dim pstrDesc         As String
   Dim pstrSource       As String
   Dim plngErrNum       As Long
   Dim pbolFoundIt      As Boolean
   
   Const UNC_STRING_TOO_SHORT = 1
   Const UNC_OPENENUM_FAILED = 2
   Const UNC_ENUM_FAILED = 3
   Const UNC_OPENENUM_RTN_ZERO = 4
   Const UNC_NO_MATCHES = 5
   Const UNC_NOT_REMOTE_DRIVE = 6
   
   On Error Resume Next
   pstrSource = App.Title & "." & "GetUNCName"
   
   If Len(Trim$(xstrPassed)) < 2 Then
      pstrDesc = "Passed a string of less than 2 characters!"
      plngErrNum = BASE_ERR_NUM + UNC_STRING_TOO_SHORT
      GoTo ExitBeforeOpen
   End If
   
   If GetDriveType(Left$(xstrPassed, 2)) <> DRIVE_REMOTE Then
      pstrDesc = "Passed a local, not a remote drive!"
      plngErrNum = BASE_ERR_NUM + UNC_NOT_REMOTE_DRIVE
      GoTo ExitBeforeOpen
   End If


   ' ---------------------------------------------
   ' Divide the passed passed string into 2
   '     strings (Drive+colon, and remaining)
   ' ---------------------------------------------
   On Error Resume Next
   pstrStart = Left$(xstrPassed, 2)
   pstrRemaining = Mid$(xstrPassed, 3)
   
   
   ' ---------------------------------------------
   ' Open the enumeration handle
   ' ---------------------------------------------
   If WNetOpenEnum(RESOURCE_CONNECTED, _
                   RESOURCETYPE_ANY, _
                   RESOURCEUSAGE_ALL, _
                   ByVal 0&, _
                   hEnum) = NO_ERROR Then
                   
      plngCount = RESOURCE_ENUM_ALL
      
      ReDim pabytBuf(0 To &H10000) As Byte
      plngBufSize = UBound(pabytBuf)
      
      ' ------------------------------------------
      ' Enumerate the network resources
      ' ------------------------------------------
      plngRtn = WNetEnumResource(hEnum, plngCount, _
                                 pabytBuf(0), plngBufSize)
      
      ' ------------------------------------------
      ' If the buffer for enumeration wasn't big
      '     enough, increase the size
      ' ------------------------------------------
      If plngRtn = ERROR_MORE_DATA Then
          plngCount = RESOURCE_ENUM_ALL
          plngBufSize = plngBufSize * 2
          ReDim pabytBuf(0 To plngBufSize) As Byte
          plngRtn = WNetEnumResource(hEnum, plngCount, _
                                     pabytBuf(0), plngBufSize)
      End If
      
      ' ------------------------------------------
      ' If successful, then parse out the
      '     individual resources
      ' ------------------------------------------
      If plngRtn = NO_ERROR Then
         pbolFoundIt = False
         If plngCount > 0 Then
            ReDim patypNetApi(0 To plngCount - 1) As NETRESOURCE
            ReDim patypNet(0 To plngCount - 1) As NETRESOURCE_REAL
            CopyMem patypNetApi(0), pabytBuf(0), Len(patypNetApi(0)) * plngCount
            
            ' ------------------------------------
            ' For each of the returned resources...
            ' ------------------------------------
            For plngLoop = 0 To plngCount - 1
               
               ' ---------------------------------
               ' Get the local (pathed) name
               ' ---------------------------------
               If patypNetApi(plngLoop).pLocalName Then
                  plngLen = lstrlen(patypNetApi(plngLoop).pLocalName)
                  patypNet(plngLoop).sLocalName = Space$(plngLen)
                  CopyMem ByVal patypNet(plngLoop).sLocalName, ByVal patypNetApi(plngLoop).pLocalName, plngLen
               End If
               
               ' ---------------------------------
               ' Get the remote (UNC) name
               ' ---------------------------------
               If patypNetApi(plngLoop).pRemoteName Then
                  plngLen = lstrlen(patypNetApi(plngLoop).pRemoteName)
                  patypNet(plngLoop).sRemoteName = Space$(plngLen)
                  CopyMem ByVal patypNet(plngLoop).sRemoteName, ByVal patypNetApi(plngLoop).pRemoteName, plngLen
               End If
               
               ' ---------------------------------
               ' Compare the local name to the
               '     passed name, if equal, return
               '     that value
               ' ---------------------------------
               On Error Resume Next
               If UCase$(patypNet(plngLoop).sLocalName) = UCase$(pstrStart) Then
                  GetUNCName = patypNet(plngLoop).sRemoteName & pstrRemaining
                  pbolFoundIt = True
                  Exit For
               End If
            
            Next plngLoop
         Else
            pstrDesc = "API Call WNetEnumResource returned a count of zero." _
                              & ", " & ReturnApiErrString(Err.LastDllError)
            plngErrNum = BASE_ERR_NUM + UNC_OPENENUM_RTN_ZERO
            GoTo ExitAfterOpen
         End If
      
      Else
         pstrDesc = "API Call WNetEnumResource failed with a buffer size of " _
                           & CStr(plngBufSize) & ", " & _
                           ReturnApiErrString(Err.LastDllError)
         plngErrNum = BASE_ERR_NUM + UNC_ENUM_FAILED
         GoTo ExitAfterOpen
      End If
      
   Else
      pstrDesc = "API Call WNetOpenEnum failed, " & _
                 ReturnApiErrString(Err.LastDllError)
      plngErrNum = BASE_ERR_NUM + UNC_OPENENUM_FAILED
      GoTo ExitBeforeOpen
   End If
   
   GoTo JustExit
   
ExitAfterOpen:
   If hEnum Then
       plngLoop = WNetCloseEnum(hEnum)
   End If
   
   If IsCompiled() = False Then
      On Error Resume Next
   Else
      On Error GoTo 0
   End If
   Err.Raise plngErrNum, pstrSource, pstrDesc
   Exit Function
   
ExitBeforeOpen:
   If IsCompiled() = False Then
      On Error Resume Next
   Else
      On Error GoTo 0
   End If
   Err.Raise plngErrNum, pstrSource, pstrDesc
   Exit Function
   
JustExit:
   On Error Resume Next
   If pbolFoundIt = False Then
      pstrDesc = "Did not find a UNC match for the drive, " & pstrStart
      plngErrNum = BASE_ERR_NUM + UNC_NO_MATCHES
      
      If IsCompiled() = False Then
         On Error Resume Next
      Else
         On Error GoTo 0
      End If
      Err.Raise plngErrNum, pstrSource, pstrDesc
   End If

End Function


