VERSION 1.0 CLASS
BEGIN
  MultiUse = 0   'False
END
Attribute VB_Name = "UncConversion"
Attribute VB_Creatable = True
Attribute VB_Exposed = True
Attribute VB_HelpID = 1
Attribute VB_Description = "Convert back and forth between UNC and drive letters"
' ------------------------------------------------------
' UNCConvt.CLS -- Copyright (c) Slightly Tilted Software
' By:       L.J. Johnson
' Date:     10-13-1996
' Comments: Class module with one exposed method,
'           GetUNCName.
' ------------------------------------------------------
Option Explicit

Private Const BASE_ERR_NUM = vbObjectError + 1048

Private Const NO_ERROR = 0&
Private Const ERROR_MORE_DATA = 234&

Private Declare Sub CopyMem _
    Lib "kernel32" Alias "RtlMoveMemory" _
    (lpTo As Any, _
     lpFrom As Any, _
     ByVal plngCount As Long)
Private Declare Function lstrlen _
    Lib "kernel32" Alias "lstrlenA" _
    (ByVal sText As Any) As Long
Private Declare Function WNetOpenEnum _
   Lib "mpr.dll" Alias "WNetOpenEnumA" _
   (ByVal dwScope As Long, _
    ByVal dwType As Long, _
    ByVal dwUsage As Long, _
    lpNetResource As Any, _
    lphEnum As Long) As Long
Private Declare Function WNetEnumResource _
   Lib "mpr.dll" Alias "WNetEnumResourceA" _
   (ByVal hEnum As Long, _
    lpcCount As Long, _
    lpBuffer As Any, _
    lpBufferSize As Long) As Long
Private Declare Function WNetCloseEnum _
   Lib "mpr.dll" _
   (ByVal hEnum As Long) As Long

Private Const RESOURCE_CONNECTED = &H1&
Private Const RESOURCETYPE_ANY = &H0&
Private Const RESOURCEUSAGE_ALL = &H0&
Private Const RESOURCE_ENUM_ALL = &HFFFF&

Private Type NETRESOURCE
    dwScope          As Long
    dwType           As Long
    dwDisplayType    As Long
    dwUsage          As Long
    pLocalName       As Long
    pRemoteName      As Long
    pComment         As Long
    pProvider        As Long
End Type

Private Type NETRESOURCE_REAL
    dwScope          As Long
    dwType           As Long
    dwDisplayType    As Long
    dwUsage          As Long
    sLocalName       As String
    sRemoteName      As String
    sComment         As String
    sProvider        As String
End Type

Private Const DRIVE_REMOTE = 4&
Private Declare Function GetDriveType _
   Lib "kernel32" Alias "GetDriveTypeA" _
   (ByVal nDrive As String) As Long

' ------------------------------------------------
' Passed either a UNC or a drive-letter path
'     (with or without a filename), determine if
'     that path is valid
' Returns TRUE if valid, FALSE otherwise
' ------------------------------------------------
Private Function IsValidPath(xstrPassed As String) As Boolean
   Dim pstrRtn          As String
   
   On Error Resume Next
   pstrRtn = Dir(xstrPassed)
   If pstrRtn = "" Then
      If Right$(xstrPassed, 1) <> "\" Then
         xstrPassed = xstrPassed & "\."
      Else
         xstrPassed = xstrPassed & "."
      End If
      pstrRtn = Dir(xstrPassed)
      If pstrRtn = "" Then
         IsValidPath = False
      Else
         IsValidPath = True
      End If
   Else
      IsValidPath = True
   End If
   On Error GoTo 0
   
End Function

' ------------------------------------------------
'
' ------------------------------------------------
Public Function GetUNCName(xstrPassed As String, _
                  Optional xvntCheckPath As Variant) As String
Attribute GetUNCName.VB_HelpID = 2
Attribute GetUNCName.VB_Description = "Convert either Letter-path or UNC-path to the opposite. Second parameter is optional, and if set to TRUE checks if the full path or path/filename is valid."
   Dim plngRtn          As Long
   Dim hEnum            As Long
   Dim plngCount        As Long
   Dim plngLoop         As Long
   Dim plngLen          As Long
   Dim plngBufSize      As Long
   Dim patypNetApi()    As NETRESOURCE
   Dim patypNet()       As NETRESOURCE_REAL
   Dim pabytBuf()       As Byte
   Dim pstrStart        As String
   Dim pstrRemaining    As String
   Dim plngPos          As Long
   Dim pstrDesc         As String
   Dim pstrSource       As String
   Dim plngErrNum       As Long
   Dim pbolRtnUnc       As Boolean
   Dim pbolFoundIt      As Boolean
   Dim pbolCheckPath    As Boolean
   Dim pstrTmpFilePath  As String
   
   Const UNC_STRING_TOO_SHORT = 1
   Const UNC_OPENENUM_FAILED = 2
   Const UNC_ENUM_FAILED = 3
   Const UNC_OPENENUM_RTN_ZERO = 4
   Const UNC_NO_UNC_MATCHES = 5
   Const UNC_NO_PATH_MATCHES = 6
   Const UNC_NOT_VALID_PATH = 7
   Const UNC_NOT_VALID_UNC = 8
   Const UNC_NOT_VALID_CHECKED_PATH = 9
   Const UNC_NOT_REMOTE_DRIVE = 10
   
   On Error Resume Next
   pstrSource = App.Title & "." & "GetUNCName"
   xstrPassed = Trim$(xstrPassed)
   
   ' xxx
   If IsMissing(xvntCheckPath) = False Then
      pbolCheckPath = CBool(xvntCheckPath)
   Else
      pbolCheckPath = False
   End If
   
   ' xxx
   If Left$(xstrPassed, 2) = "\\" Then
      pbolRtnUnc = False
   Else
      pbolRtnUnc = True
   End If
   
   ' Make sure not less than 2 characters passed for
   '     path, or no less than 5 chacters for UNC
   If pbolRtnUnc = True Then
      If Len(xstrPassed) < 2 Then
         pstrDesc = "Passed a string of less than 2 characters!"
         plngErrNum = BASE_ERR_NUM + UNC_STRING_TOO_SHORT
         GoTo ExitBeforeOpen
      End If
      
      If Asc(UCase$(Left(xstrPassed, 1))) < 65 Or _
         Asc(UCase$(Left(xstrPassed, 1))) > 90 Then
         pstrDesc = "First character of path must be an alpha character!"
         plngErrNum = BASE_ERR_NUM + UNC_NOT_VALID_PATH
         GoTo ExitBeforeOpen
      ElseIf Mid$(xstrPassed, 2, 1) <> ":" Then
         pstrDesc = "Second character of path must be a colon!"
         plngErrNum = BASE_ERR_NUM + UNC_NOT_VALID_PATH
         GoTo ExitBeforeOpen
      End If
   
      If GetDriveType(Left$(xstrPassed, 2)) <> DRIVE_REMOTE Then
         pstrDesc = "Passed a local, not a remote drive!"
         plngErrNum = BASE_ERR_NUM + UNC_NOT_REMOTE_DRIVE
         GoTo ExitBeforeOpen
      End If
   Else
      If Len(xstrPassed) < 5 Then
         pstrDesc = "Passed a string of less than 5 characters!"
         plngErrNum = BASE_ERR_NUM + UNC_STRING_TOO_SHORT
         GoTo ExitBeforeOpen
      End If
      
      If Mid$(xstrPassed, 1, 2) <> "\\" Then
         pstrDesc = "First 2 character2 of UNC path must be '\\'!"
         plngErrNum = BASE_ERR_NUM + UNC_NOT_VALID_UNC
         GoTo ExitBeforeOpen
      End If
   End If
   
   ' Divide the passed passed string into 2
   '     strings (Drive+colon, and remaining)
   '     depending on whether or not passed
   '     UNC
   On Error Resume Next
   If pbolRtnUnc = True Then
      pstrStart = Left$(xstrPassed, 2)
      pstrRemaining = Mid$(xstrPassed, 3)
   Else
      On Error Resume Next
      plngPos = InStr(3, xstrPassed, "\")
      If plngPos > 0 Then
         plngPos = InStr(plngPos + 1, xstrPassed, "\")
         If plngPos > 0 Then
            pstrStart = Left(xstrPassed, plngPos - 1)
            pstrRemaining = Mid$(xstrPassed, plngPos)
         Else
            pstrStart = xstrPassed
            pstrRemaining = ""
         End If
      End If
   End If
   
   ' Open the enumeration handle
   If WNetOpenEnum(RESOURCE_CONNECTED, _
                   RESOURCETYPE_ANY, _
                   RESOURCEUSAGE_ALL, _
                   ByVal 0&, _
                   hEnum) = NO_ERROR Then
                   
      plngCount = RESOURCE_ENUM_ALL
      
      ReDim pabytBuf(0 To &H10000) As Byte
      plngBufSize = UBound(pabytBuf)
      
      ' Enumerate the network resources
      plngRtn = WNetEnumResource(hEnum, plngCount, _
                                 pabytBuf(0), plngBufSize)
      
      ' If the buffer for enumeration wasn't big
      '     enough, increase the size
      If plngRtn = ERROR_MORE_DATA Then
          plngCount = RESOURCE_ENUM_ALL
          plngBufSize = plngBufSize * 2
          ReDim pabytBuf(0 To plngBufSize) As Byte
          plngRtn = WNetEnumResource(hEnum, plngCount, _
                                     pabytBuf(0), plngBufSize)
      End If
      
      ' If successful, then parse out the
      '     individual resources
      If plngRtn = NO_ERROR Then
         pbolFoundIt = False
         If plngCount > 0 Then
            ReDim patypNetApi(0 To plngCount - 1) As NETRESOURCE
            ReDim patypNet(0 To plngCount - 1) As NETRESOURCE_REAL
            CopyMem patypNetApi(0), pabytBuf(0), Len(patypNetApi(0)) * plngCount
            
            ' For each of the returned resources...
            For plngLoop = 0 To plngCount - 1
               
               ' Get the local (pathed) name
               If patypNetApi(plngLoop).pLocalName Then
                  plngLen = lstrlen(patypNetApi(plngLoop).pLocalName)
                  patypNet(plngLoop).sLocalName = Space$(plngLen)
                  CopyMem ByVal patypNet(plngLoop).sLocalName, ByVal patypNetApi(plngLoop).pLocalName, plngLen
               End If
               
               ' Get the remote (UNC) name
               If patypNetApi(plngLoop).pRemoteName Then
                  plngLen = lstrlen(patypNetApi(plngLoop).pRemoteName)
                  patypNet(plngLoop).sRemoteName = Space$(plngLen)
                  CopyMem ByVal patypNet(plngLoop).sRemoteName, ByVal patypNetApi(plngLoop).pRemoteName, plngLen
               End If
               
               ' Compare the local name to the
               '     passed name, if equal, return
               '     that value
               On Error Resume Next
               If pbolRtnUnc = True Then
                  If UCase$(patypNet(plngLoop).sLocalName) = UCase$(pstrStart) Then
                     pstrTmpFilePath = patypNet(plngLoop).sRemoteName & pstrRemaining
                     GetUNCName = pstrTmpFilePath
                     pbolFoundIt = True
                     Exit For
                  End If
               Else
                  If UCase$(patypNet(plngLoop).sRemoteName) = UCase$(pstrStart) Then
                     pstrTmpFilePath = patypNet(plngLoop).sLocalName & pstrRemaining
                     GetUNCName = pstrTmpFilePath
                     pbolFoundIt = True
                     Exit For
                  End If
               End If
            Next plngLoop
         Else
            pstrDesc = "API Call WNetEnumResource returned a count of zero." _
                              & ", " & ReturnApiErrString(Err.LastDllError)
            plngErrNum = BASE_ERR_NUM + UNC_OPENENUM_RTN_ZERO
            GoTo ExitAfterOpen
         End If
      
      Else
         pstrDesc = "API Call WNetEnumResource failed with a buffer size of " _
                           & CStr(plngBufSize) & ", " & _
                           ReturnApiErrString(Err.LastDllError)
         plngErrNum = BASE_ERR_NUM + UNC_ENUM_FAILED
         GoTo ExitAfterOpen
      End If
      
   Else
      pstrDesc = "API Call WNetOpenEnum failed, " & _
                 ReturnApiErrString(Err.LastDllError)
      plngErrNum = BASE_ERR_NUM + UNC_OPENENUM_FAILED
      GoTo ExitBeforeOpen
   End If
   
   ' If user wants, check to see if valid path
   If pbolCheckPath = True Then
      If IsValidPath(pstrTmpFilePath) = False Then
         pstrDesc = "The Path/UNC conversion was successful, but the complete path is invalid!"
         plngErrNum = BASE_ERR_NUM + UNC_NOT_VALID_CHECKED_PATH
         GetUNCName = ""
         GoTo ExitAfterOpen
      End If
   End If
   
   GoTo JustExit
   
ExitAfterOpen:
   If hEnum Then
       plngLoop = WNetCloseEnum(hEnum)
   End If
   
   If IsCompiled() = False Then
      On Error Resume Next
   Else
      On Error GoTo 0
   End If
   Err.Raise plngErrNum, pstrSource, pstrDesc
   Exit Function
   
ExitBeforeOpen:
   If IsCompiled() = False Then
      On Error Resume Next
   Else
      On Error GoTo 0
   End If
   Err.Raise plngErrNum, pstrSource, pstrDesc
   Exit Function
   
JustExit:
   If pbolFoundIt = False Then
      If pbolRtnUnc = True Then
         pstrDesc = "Did not find a UNC match for the drive, " & pstrStart
         plngErrNum = BASE_ERR_NUM + UNC_NO_UNC_MATCHES
      Else
         pstrDesc = "Did not find a PATH match for the UNC string, " & pstrStart
         plngErrNum = BASE_ERR_NUM + UNC_NO_PATH_MATCHES
      End If
      
      If IsCompiled() = False Then
         On Error Resume Next
      Else
         On Error GoTo 0
      End If
      Err.Raise plngErrNum, pstrSource, pstrDesc
   End If

End Function
